package dk.skat.security.providers.atn;

import dk.skat.begrebsmodel.x2009.x01.x15.*;
import dk.skat.begrebsmodel.xml.schemas.kontekst.x2007.x05.x31.HovedOplysningerDocument;
import dk.skat.begrebsmodel.xml.schemas.kontekst.x2007.x05.x31.HovedOplysningerSvarDocument;
import dk.skat.begrebsmodel.xml.schemas.kontekst.x2007.x05.x31.HovedOplysningerType;
import dk.skat.begrebsmodel.xml.schemas.kontekst.x2007.x05.x31.SvarReaktionType;
import dk.skat.ics.security.EORIVirksomhedSamlingHentService;
import dk.skat.ics.security.EORIVirksomhedSamlingHentServicePortType;
import dk.skat.ics.security.EORIVirksomhedSamlingHentService_Impl;
import net.sf.ehcache.Cache;
import net.sf.ehcache.CacheManager;
import net.sf.ehcache.Element;
import weblogic.logging.NonCatalogLogger;

import javax.xml.rpc.ServiceException;
import java.rmi.RemoteException;
import java.util.List;

/**
 * EORI Number Validator Class.
 * <p/>
 * Makes call to the PS_EORI_VirksomhedSamlingHent service
 *
 * @author Christopher Derek Curry, KrisPoint A/S, cdcurry@krispoint.com
 * @since 1.0
 */
public class EORINumberValidator {

    /**
     * Logger
     */
    private final static NonCatalogLogger logger = new NonCatalogLogger(EORINumberValidator.class.getName());


    private static final CacheManager cacheManager = CacheManager.create("ics-security.xml");

    private static Cache getCache() {
        String cacheName = "EORIVirksomhedSamlingHentServiceCache";
        return cacheManager.getCache(cacheName);
    }


    public static boolean validateEORINumber(String eori, String endPoint) {
        boolean result = false;
        if (getCache().isKeyInCache(eori)) {
            logger.notice("EORI number "+ eori + " is in cache.");
            Element e = getCache().get(eori);
            if (e != null) {
                  return (Boolean) e.getObjectValue();
            } else {
                logger.notice("Element expired or evicted. Validating EORI number " + eori + " again.");
                boolean r = callValidateEORINumber(eori, endPoint);
                Element element = new Element(eori, r);
                getCache().put(element);
                result = r;
            }
        } else {
            logger.notice("Validating EORI number: " + eori);
            boolean r = callValidateEORINumber(eori, endPoint);
            Element element = new Element(eori, r);
            getCache().put(element);
            result = r;
        }
        return result;
    }


    public static boolean callValidateEORINumber(String eori, String endPoint) {

        // Assume false.
        boolean result = false;

        try {
            EORIVirksomhedSamlingHentService service = new EORIVirksomhedSamlingHentService_Impl(endPoint);
            EORIVirksomhedSamlingHentServicePortType port = service.getEORIVirksomhedSamlingHentServicePortType();
            EORIVirksomhedSamlingHentIDocument input = EORIVirksomhedSamlingHentIDocument.Factory.newInstance();
            EORIVirksomhedSamlingHentIType eoriVirksomhedSamlingHentI = input.addNewEORIVirksomhedSamlingHentI();
            EORIVirksomhedSamlingHentIType.EORINummerSamling eoriNummerSamling = eoriVirksomhedSamlingHentI.addNewEORINummerSamling();
            eoriNummerSamling.addØkonomiskOperatørEORINummer(eori);
            KontekstType kontekstTypeInput = KontekstType.Factory.parse(
                    HovedOplysningerDocumentFactory.getHovedOplysningerDocument("ICSAuthenticationProvider",
                            "SecurityProvider", "SecurityProvider").toString());
            eoriVirksomhedSamlingHentI.setKontekst(kontekstTypeInput);
            logger.notice("EORIVirksomhedSamlingHentService [INPUT]: " + input);
            EORIVirksomhedSamlingHentODocument output = port.getEORIVirksomhedSamlingHent(input);
            logger.notice("EORIVirksomhedSamlingHentService [OUTPUT]" + output);
            KontekstType kontekstTypeOutput = output.getEORIVirksomhedSamlingHentO().getKontekst();
            HovedOplysningerSvarDocument svarDoc = HovedOplysningerSvarDocument.Factory.parse(kontekstTypeOutput + "");
            List<SvarReaktionType> reaktionList = svarDoc.getHovedOplysningerSvar().getSvarReaktionList();
            for (SvarReaktionType svarType : reaktionList) {
                if (svarType.isSetFejl()) {
                    // If there is an error, then bail out of here and respond as if EORI number does not exist.
                    logger.notice("EORIVirksomhedSamlingHentService reports and error. Will respond as if EORI number "
                            + eori + " does not exist. " +
                            "The service responded with error (first in list): "
                            + svarType.getFejl().getFejlNummer() + " - " + svarType.getFejl().getFejlTekst() +
                    ". There may be additional errors.");
                    return false;
                }
            }

            if (output.getEORIVirksomhedSamlingHentO().getEORIVirksomhedSamling() == null) {
                return false;
            }

            if (output.getEORIVirksomhedSamlingHentO().getEORIVirksomhedSamling().sizeOfEORIVirksomhedArray() == 1) {
                EORIVirksomhedSamlingHentOType.EORIVirksomhedSamling.EORIVirksomhed resultEoriVirk = output.getEORIVirksomhedSamlingHentO().getEORIVirksomhedSamling().getEORIVirksomhedArray(0);
                if (eori.equals(resultEoriVirk.getØkonomiskOperatørEORINummer())) {
                    return true;
                }
            }

        } catch (RemoteException e) {
            logger.error("Error while validating EORI number " + eori + ". Will validate to false. RemoteException: ", e);
        } catch (ServiceException e2) {
            logger.error("Error while validating EORI number " + eori + ".  Will validate to false. ServiceException:", e2);
        } catch (Exception e3) {
            logger.error("Error while validating EORI number " + eori + ". Will validate to false. Exception:", e3);
        }

        return result;

    }


    public static void main(String[] args) {
        boolean x = validateEORINumber("DK82001417", "http://sktdics04esb01.csc.dk:7013/EORI/EORIVirksomhedSamlingHentService?wsdl");
    }


}
