package dk.skat.security.providers.atn;

import java.io.IOException;
import java.security.Principal;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.LoginException;

import dk.skat.ics.security.ExtendedIdentityHelper;
import weblogic.logging.NonCatalogLogger;
import weblogic.security.principal.WLSGroupImpl;
import weblogic.security.principal.WLSUserImpl;
import weblogic.security.spi.WLSUser;

/**
 * Login Module for AuthenticationProvider
 *
 * @author Christopher Derek Curry, KrisPoint A/S, cdcurry@krispoint.com
 * @since 1.0
 *        <p/>
 *        See http://download.oracle.com/docs/cd/E12840_01/wls/docs103/dvspisec/atn.html
 */
final public class LoginModule implements javax.security.auth.spi.LoginModule {

    private static final NonCatalogLogger logger = new NonCatalogLogger(LoginModule.class.getName());

    /**
     * Subject that is to be authenticated.
     */
    private Subject subject;

    /**
     * Call back handler
     */
    private CallbackHandler callbackHandler;

    /**
     * Result of assertion/login process: Principal for this user.
     */
    private WLSUser wlsUser;

    private String serviceUrl;

    private String staticGroups;

    private String skatCVRNumbers;

    private Boolean validateEoriNumber = false;

    private static final String nameCallBackPrompt = "username: ";

    private ArrayList<WLSGroupImpl> listGroups = new ArrayList<WLSGroupImpl>();

    /**
     * Transform a comma seperated list to a list of WLSGroupImpl objects.
     * <p/>
     * This method modifies the <code>listGroups</code> object.
     *
     * @param list Comma seperated list as a string
     */
    private void addWLSGroups(String list) {
        String group;
        // Only process list if there is a valid string
        if (list != null && list.length() > 0) {
            StringTokenizer st = new StringTokenizer(list, ",");
            while (st.hasMoreElements()) {
                group = st.nextToken();
                listGroups.add(new WLSGroupImpl(group));
            }
        }

    }


    private String getValueFromName(String list, String identifier) {
        String group;
        // Only process list if there is a valid string
        if (list != null && list.length() > 0) {
            StringTokenizer st = new StringTokenizer(list, ",");
            while (st.hasMoreElements()) {
                group = st.nextToken();
                if (group.startsWith(identifier)) {
                    return group.substring(identifier.length());
                }
            }
        }
        return null;

    }


    private boolean loginSucceeded;

    /**
     * Determine whether this is a assert identity job or not.
     */
    private boolean isIdentityAssertion;

    /**
     * @see javax.security.auth.spi.LoginModule#initialize(javax.security.auth.Subject, javax.security.auth.callback.CallbackHandler, java.util.Map, java.util.Map)
     */
    public void initialize(Subject subject, CallbackHandler callbackHandler, Map sharedState, Map options) {
        this.subject = subject;
        this.callbackHandler = callbackHandler;
        isIdentityAssertion =
                "true".equalsIgnoreCase((String) options.get("IdentityAssertion"));
        serviceUrl = (String) options.get("serviceUrl");
        staticGroups = (String) options.get("staticGroups");
        skatCVRNumbers = (String) options.get("skatCVRNumbers");
        validateEoriNumber = (Boolean) options.get("validateEoriNumber");
        getSKATCVRNumbers();
        
    }

    /**
     * @see javax.security.auth.spi.LoginModule#login()
     */
    public boolean login() throws LoginException {
        if (isIdentityAssertion) {
            return assertLogin();
        } else {
            return authenticateLogin();
        }

    }

    private static final String NAME_PREFIX = "SKATEORI_";


    /**
     * Authenticate login
     *
     * @return true if authentication went well except if the provided name is null or does not match the
     *         required <code>NAME_PREFIX</code>
     * @throws LoginException TODO
     */
    private boolean authenticateLogin() throws LoginException {
        NameCallback nameCallback = new NameCallback(nameCallBackPrompt);
        getCallback(nameCallback);

        String user = nameCallback.getName();
        assertEmptyString(user);

        if (!user.startsWith(NAME_PREFIX)) {
            loginSucceeded = false;
            return loginSucceeded;
        }
        String theGroups = user.substring(NAME_PREFIX.length(), user.length()) + ",Administrators";

        addWLSGroups(theGroups);
        wlsUser = new WLSUserImpl(user);
        loginSucceeded = true;
        logger.notice("Authenticated: " + user);
        return loginSucceeded;

    }

    /**
     * Assert user login.
     *
     * @return true if assertion went well except if the provided name is null or does not match the
     *         required <code>NAME_PREFIX</code>
     * @throws LoginException 
     */
    private boolean assertLogin() throws LoginException {
        NameCallback nameCallback = new NameCallback(nameCallBackPrompt);
        getCallback(nameCallback);
        String user = nameCallback.getName();
        assertEmptyString(user);

        // Only assert string that have a specific prefix!
        if (!user.startsWith(NAME_PREFIX)) {
            loginSucceeded = false;
            return loginSucceeded;
        }
        // 'Chop chop' comma separated string and turn tokens into WLSGroups
        String theGroups = user.substring(NAME_PREFIX.length(), user.length());
        logger.debug("Received this set of tokens '" + theGroups + "'");
        addWLSGroups(theGroups);

        // Add static WLS Groups if defined in "Provider Specific" section.
        if (staticGroups != null && staticGroups.length() >0 ) {
            addWLSGroups(staticGroups);
        }
        // Validate EORI number - if provided via callback.
        String eoriNumber = getValueFromName(theGroups, ExtendedIdentityHelper.ICS_IDENTITY_EORI);
        String cvrNumber = getValueFromName(theGroups, ExtendedIdentityHelper.ICS_IDENTITY_CVR);
        if (eoriNumber != null && eoriNumber.length() > 0 && validateEoriNumber.booleanValue()) {
            boolean valid = EORINumberValidator.validateEORINumber(eoriNumber, serviceUrl);
            if (!valid) {
                logger.info("Cannot login '" + user + "' as EORI number could not be validated!");
                loginSucceeded = false;
                return loginSucceeded;
            }
        }
        // Check if user is Employee in DK company
        if (!map.containsKey(cvrNumber) && eoriNumber == null) {
            addWLSGroups("AUTH_LEVEL_4");
        } else
        // Check if user is SKAT Employee in DK company
        if (map.containsKey(cvrNumber) && eoriNumber == null) {
            addWLSGroups("AUTH_LEVEL_6");
        } else
        // Check if user is Employee in EORI company
        if (eoriNumber != null && eoriNumber.length() > 0) {
            addWLSGroups("AUTH_LEVEL_4");
        }
        // Set the name received in the call back as WLSUser
        wlsUser = new WLSUserImpl(user);
        loginSucceeded = true;
        logger.notice("Asserted: " + user);
        return loginSucceeded;
    }


    /**
     * Simple helper method to assert strings.
     *
     * @param s String
     * @throws IllegalArgumentException If String is empty or NULL.
     */
    public static void assertEmptyString(String s) throws IllegalArgumentException {
        if ((s == null) || "".equals(s.trim())) {
            throw new IllegalArgumentException("The provided value cannot be null or empty.");
        }

    }

    /**
     * @see javax.security.auth.spi.LoginModule#commit()
     */
    public boolean commit() throws LoginException {
        try {
            if (loginSucceeded) {

                Set<Principal> principals = subject.getPrincipals();
                for (WLSGroupImpl s : listGroups) {
                    principals.add(s);
                    logger.debug("Adding group '" + s.getName() + "' for user '" + wlsUser.getName() + "'.");
                }
                principals.add(wlsUser);
            }
            logger.debug("Committed login for user '" + wlsUser + "'");
            return true;

        } catch (Exception e) {
            logger.error("Error: ", e);
            return false;
        }
    }

    /**
     * @see javax.security.auth.spi.LoginModule#abort()
     */
    public boolean abort() throws LoginException {
        try {
            Set<Principal> principals = subject.getPrincipals();

            principals.remove(wlsUser);
            for (WLSGroupImpl s : listGroups) {
                principals.remove(s);
            }
            return true;

        } catch (Exception e) {
            logger.error("Error: ", e);
            return false;
        }
    }

    /**
     * @see javax.security.auth.spi.LoginModule#logout()
     */
    public boolean logout() throws LoginException {
        try {
            Set<Principal> principals = subject.getPrincipals();

            principals.remove(wlsUser);
            for (WLSGroupImpl s : listGroups) {
                principals.remove(s);
            }
            return true;
        } catch (Exception e) {
            logger.error("Error: ", e);
            return false;
        }
    }


    private void getCallback(Callback... callbacks) throws LoginException {
        try {
            callbackHandler.handle(callbacks);
        } catch (IOException e) {
            throw new LoginException(e.getMessage());
        } catch (UnsupportedCallbackException e) {
            throw new LoginException(e.getMessage());
        }
    }

    private static Map<String, String> map = null;

    private void getSKATCVRNumbers() {
        logger.info("Processing list of SKAT CVR Numbers: " + skatCVRNumbers);
        if (map == null) {
            map = new HashMap<String, String>();
            StringTokenizer st = new StringTokenizer(skatCVRNumbers, ",");
            while (st.hasMoreElements()) {
                String value = st.nextToken();
                map.put(value, value);
                logger.notice("Processing CVR: " + value);
            }
        }

    }
  
}
