package dk.skat.security.providers.roles;

import java.util.*;
import javax.security.auth.Subject;


import dk.skat.ics.security.ExtendedIdentityHelper;
import dk.skat.ics.security.IdentityInformation;
import dk.skat.security.providers.atn.EORINumberValidator;
import dk.skat.security.providers.validators.VirksomhedValidator;
import weblogic.management.security.ProviderMBean;
import weblogic.security.service.ContextHandler;
import weblogic.security.spi.ApplicationInfo;
import weblogic.security.spi.DeployableRoleProviderV2;
import weblogic.security.spi.DeployRoleHandle;
import weblogic.security.spi.Resource;
import weblogic.security.spi.RoleMapper;
import weblogic.security.spi.SecurityServices;
import weblogic.security.spi.VersionableApplicationProvider;
import weblogic.security.SubjectUtils;
import weblogic.logging.NonCatalogLogger;


/**
 * ICS Role Mapping Provider
 *
 */
public final class ICSRoleMapperProviderImpl
        implements DeployableRoleProviderV2, RoleMapper, VersionableApplicationProvider {


    /**
     * Logger
     */
    private final static NonCatalogLogger logger = new NonCatalogLogger(ICSRoleMapperProviderImpl.class.getName());

    private String description; // a description of this provider

    private static final Map NO_ROLES = Collections.unmodifiableMap(new HashMap(1)); // used when no roles are found

    private boolean doDebug;

    private boolean validateCVR;

    private String eoriVirksomhedValiderServiceEndpoint;

    private Boolean validateEoriNumber = false;

    private String eoriVirksomhedSamlingHentEndpoint;


    /**
     * Initialize the simple sample role mapper.
     * <p/>
     * It either creates or re-opens the file containing the
     * simple sample role mapper's role definitions.
     *
     * @param mbean    A ProviderMBean that holds the simple sample role mapper's
     *                 configuration data.  This mbean must be an instance of the simple sample
     *                 role mapper's mbean.
     * @param services The SecurityServices gives access to the auditor
     *                 so that the provider can to post audit events.
     *                 The simple sample role mapper doesn't use this parameter.
     * @see weblogic.security.spi.SecurityProvider
     */
    public void initialize(ProviderMBean mbean, SecurityServices services) {

        // Cast the mbean from a generic ProviderMBean to a SimpleSampleRoleMapperMBean.
        ICSRoleMappingProviderMBean myMBean = (ICSRoleMappingProviderMBean) mbean;

        // Set the description to the simple sample role mapper's mbean's description and version
        description = myMBean.getDescription() + "\n" + myMBean.getVersion();

        doDebug = myMBean.getDebugRoleMapping();
        validateCVR = myMBean.getValidateCVR();
        eoriVirksomhedValiderServiceEndpoint = myMBean.getEORIVirksomhedValiderService();
        validateEoriNumber = myMBean.getValidateEORINumber();
        eoriVirksomhedSamlingHentEndpoint = myMBean.getEORIVirksomhedSamlingHent();
    }
    
    /**
     * Get the simple sample role mapper's description.
     *
     * @return A String containing a brief description of the simple sample role mapper.
     * @see weblogic.security.spi.SecurityProvider
     */
    public String getDescription() {
        return description;
    }



    /**
     * Shutdown the simple sample role mapper.
     * <p/>
     * A no-op.
     *
     * @see weblogic.security.spi.SecurityProvider
     */
    public void shutdown() {
    }

    /**
     * Gets the simple sample role mapper provider's role mapper object.
     *
     * @return The simple sample role mapper provider's RoleMapper object.
     * @see weblogic.security.spi.RoleProvider
     */
    public RoleMapper getRoleMapper() {
        // Since this class implements both the DeployableRoleProvider
        // and RoleMapper interfaces, this object is the
        // role mapper object so just return "this".
        return this;
    }


    /**
     * Determines what roles the current subject is in
     * for this resource.
     *
     * @param subject  A Subject that contains the Principals that identify
     *                 the user who is trying to access the resource as well as the user's
     *                 groups.
     * @param resource The Resource the Subject is trying to access.
     * @param handler  A ContextHandler contains additional information that
     *                 may be considered in computing the roles.  This parameter is not used
     *                 by the simple sample role mapper.
     * @return a Map containing SecurityRoles identifying the computed roles.
     *         The SecurityRoles are instances of the simple sample identity asserter's
     *         SecurityRole implementation (SimpleSampleSecurityRoleImpl).
     * @see weblogic.security.spi.RoleMapper
     */
    public Map getRoles(Subject subject, Resource resource, ContextHandler handler) {

        String username = SubjectUtils.getUsername(subject);
        // Don't waste time on anonymous
        if (SubjectUtils.isUserAnonymous(subject)) {
            return NO_ROLES;
        }

        Map roles = new HashMap();
        IdentityInformation info = ExtendedIdentityHelper.getIdentityInformation(subject.getPrincipals());
        if (info.isEmployee()) {
            String role = "USERTYPE_EMPLOYEE";
            roles.put(role, new SecurityRoleImpl(role));

            if (validateCVR) {
                String cvr = info.getCvr();
                if (VirksomhedValidator.validateCVRNumber(cvr,  eoriVirksomhedValiderServiceEndpoint)) {
                    role = "EORI_TRANSACTIONS_ALLOWED";
                    roles.put(role, new SecurityRoleImpl(role));
                }
            }
        }
        if (info.isSkatEmployee()) {
            String role = "USERTYPE_SKATEMPLOYEE";
            roles.put(role, new SecurityRoleImpl(role));
        }
        if (info.isEORIUser()) {
            String role = "USERTYPE_EORIUSER";
            roles.put(role, new SecurityRoleImpl(role));
            String eoriNumber = info.getEori();
            if (validateEoriNumber) {
                if (eoriNumber != null && eoriNumber.length() > 0) {
                boolean valid = EORINumberValidator.validateEORINumber(eoriNumber, eoriVirksomhedSamlingHentEndpoint);
                if (valid) {
                    role = "EORI_TRANSACTIONS_ALLOWED";
                    roles.put(role, new SecurityRoleImpl(role));
                }
            }
            }

        }

        if (doDebug) {
            logger.notice("Received identity: '" + info.toString());
            Collection col = roles.values();
            StringBuffer sb = new StringBuffer(128);

            for (Object r : col) {
                sb.append(r.toString()).append(" ");
            }
            logger.notice("Calculated roles for '" + username + "': " + sb.toString());
        }
        if (roles.isEmpty()) {
            return NO_ROLES;
        }
        return roles;
    }

    /**
     * @see weblogic.security.spi.DeployableRoleProviderV2#startDeployRoles(weblogic.security.spi.ApplicationInfo)
     */
    public DeployRoleHandle startDeployRoles(ApplicationInfo application) {
        return null;
    }

    /**
     * @see weblogic.security.spi.DeployableRoleProviderV2#deployRole(weblogic.security.spi.DeployRoleHandle, weblogic.security.spi.Resource, String, String[])
     */
    public void deployRole(DeployRoleHandle handle, Resource resource,
                           String roleName, String[] principalNames) {
    }

    /**
     * @see weblogic.security.spi.DeployableRoleProviderV2#endDeployRoles(weblogic.security.spi.DeployRoleHandle)
     */
    public void endDeployRoles(DeployRoleHandle handle) {
    }

    /**
     * Removes the role definitions for an undeployed webapp or EJB
     * from the simple sample role mapper's properties file.
     * <p/>
     * This method removes security roles for the undeployed component
     * by scanning for the roles using the component name that is stored
     * in the SampleDeployRoleHandle.
     *
     * @param handle the same handle created from startDeployRoles()
     * @see weblogic.security.spi.DeployableRoleProviderV2
     */
    public void undeployAllRoles(DeployRoleHandle handle) {
    }

    /**
     * Removes the security roles for resources when an application is
     * deleted. Only called on the admin server which is controlling the
     * deletion process.
     * <p/>
     * This method will search through the security roles for the
     * deleted application and remove all the relevant entries.
     *
     * @param application information about the application being deleted
     * @see weblogic.security.spi.DeployableRoleProviderV2
     */
    public void deleteApplicationRoles(ApplicationInfo application) {
    }



    /**
     * Called when a new application version is created. Only called on
     * the admin server which is controlling the creation process.
     * <p/>
     * This method will search through the security roles for the
     * source application and clone all the entries for the new
     * application version. This will ensure that the new version
     * has any customized roles from the source version.
     * <p/>
     * The application identifier contains the application name
     * concatenated with the application version so that the
     * application identifier can be used as a unique key.
     *
     * @param appId       the application identifier of the newly
     *                    created application version
     * @param sourceAppId the application identifier of the
     *                    version containing the source (or seed) data for the new
     *                    application version. When no source identifier is supplied
     *                    then this is the first version of the application.
     * @see weblogic.security.spi.VersionableApplicationProvider
     */
    public void createApplicationVersion(String appId, String sourceAppId) {
    }

    /**
     * Called when an application version is deleted. Only called on
     * the admin server which is controlling the deletion process.
     * <p/>
     * This method will search through the security roles for the
     * deleted application version and remove all the entries.
     *
     * @param appId the application identifier of the
     *              deleted application version
     * @see weblogic.security.spi.VersionableApplicationProvider
     */
    public void deleteApplicationVersion(String appId) {

    }

    /**
     * Called when an non-versioned application is deleted. Only called on
     * the admin server which is controlling the deletion process.
     * <p/>
     * This method will search through the security roles for the
     * application and remove all the entries.
     *
     * @param appName the application name of the
     *                deleted application
     * @see weblogic.security.spi.VersionableApplicationProvider
     */
    public void deleteApplication(String appName) {
    }


}
