package dk.skat.security.providers.validators;

import dk.skat.begrebsmodel.x2009.x01.x15.*;
import dk.skat.begrebsmodel.xml.schemas.kontekst.x2007.x05.x31.HovedOplysningerSvarDocument;
import dk.skat.begrebsmodel.xml.schemas.kontekst.x2007.x05.x31.SvarReaktionType;
import dk.skat.ics.security.*;
import dk.skat.security.providers.atn.HovedOplysningerDocumentFactory;
import net.sf.ehcache.Cache;
import net.sf.ehcache.CacheManager;
import net.sf.ehcache.Element;
import weblogic.logging.NonCatalogLogger;

import javax.xml.rpc.ServiceException;
import java.rmi.RemoteException;
import java.security.cert.X509CRL;
import java.util.List;

/**
 *
 */
public class VirksomhedValidator {

    /**
     * Logger
     */
    private final static NonCatalogLogger logger = new NonCatalogLogger(VirksomhedValidator.class.getName());

    private static final CacheManager cacheManager = CacheManager.create("ics-security.xml");

    private static Cache getCache() {
        String cacheName = "EORIVirksomhedValiderServiceCache";
        return cacheManager.getCache(cacheName);
    }


    public static boolean validateCVRNumber(String cvr, String endPoint) {
        boolean result;
        if (getCache().isKeyInCache(cvr)) {
            logger.notice("CVR number " + cvr +" is in cache.");
            Element e = getCache().get(cvr);
            if (e != null) {
                  return (Boolean) e.getObjectValue();
            } else {
                logger.notice("Element expired or evicted. Validating CVR again.");
                boolean r = callValidateCVRNumberProxyService(cvr, endPoint);
                Element element = new Element(cvr, r);
                getCache().put(element);
                result = r;
            }
        } else {
            logger.notice("Validating CVR: " + cvr);
            boolean r = callValidateCVRNumberProxyService(cvr, endPoint);
            Element element = new Element(cvr, r);
            getCache().put(element);
            result = r;
        }
        return result;
    }


    public static boolean callValidateCVRNumberProxyService(String cvr, String endPoint) {
        // Assume false.
        boolean result = false;

        try {
            EORIVirksomhedValiderService service = new EORIVirksomhedValiderService_Impl(endPoint);
            EORIVirksomhedValiderServicePortType port = service.getEORIVirksomhedValiderServicePortType();
            EORIVirksomhedValiderIDocument eoriVirksomhedValiderIDocument = EORIVirksomhedValiderIDocument.Factory.newInstance();

            EORIVirksomhedValiderIDocument.EORIVirksomhedValiderI eoriVirksomhedValiderI =
                    eoriVirksomhedValiderIDocument.addNewEORIVirksomhedValiderI();

            EORIVirksomhedValiderIType.SECVRNummer secvrNummer = EORIVirksomhedValiderIType.SECVRNummer.Factory.newInstance();
            SENummerType seNummerType = secvrNummer.addNewVirksomhedSENummer();
            seNummerType.setStringValue(cvr);
            eoriVirksomhedValiderI.setSECVRNummer(secvrNummer);
            KontekstType kontekstTypeInput = KontekstType.Factory.parse(
                    HovedOplysningerDocumentFactory.getHovedOplysningerDocument("ICSAuthenticationProvider",
                            "SecurityProvider", "SecurityProvider").toString());
            eoriVirksomhedValiderI.setKontekst(kontekstTypeInput);
            logger.notice("EORIVirksomhedValiderService [INPUT]: " + eoriVirksomhedValiderIDocument);
            EORIVirksomhedValiderODocument output = port.getEORIVirksomhedValider(eoriVirksomhedValiderIDocument);
            logger.notice("EORIVirksomhedValiderService [OUTPUT]: " + output);

            KontekstType kontekstTypeOutput = output.getEORIVirksomhedValiderO().getKontekst();
            HovedOplysningerSvarDocument svarDoc = HovedOplysningerSvarDocument.Factory.parse(kontekstTypeOutput + "");
            List<SvarReaktionType> reaktionList = svarDoc.getHovedOplysningerSvar().getSvarReaktionList();
            for (SvarReaktionType svarType : reaktionList) {
                if (svarType.isSetFejl()) {
                    // If there is an error, then bail out of here and respond as if EORI number does not exist.
                    logger.notice("EORIVirksomhedValiderService reports an error. " +
                            "Will respond as if CVR number " + cvr + " does not exist. The service responded with error (first in list): "
                            + svarType.getFejl().getFejlNummer() + " - " + svarType.getFejl().getFejlTekst() +
                    ". There may be additional errors.");
                    return false;
                }
            }


            if (output.getEORIVirksomhedValiderO() != null) {
                EORIVirksomhedValiderOType.EORIVirksomhedValiderSvarListe liste =
                        output.getEORIVirksomhedValiderO().getEORIVirksomhedValiderSvarListe();

                List<EORIVirksomhedValiderOType.EORIVirksomhedValiderSvarListe.EORIVirksomhedValiderSvar> svarListe =
                        liste.getEORIVirksomhedValiderSvarList();
                if (svarListe != null && svarListe.size() == 1) {
                    return svarListe.get(0).getEORIErValid();
                }
            }

        } catch (RemoteException e) {
            logger.error("Error while validating CVR number " + cvr + ". Will validate to false. RemoteException:", e);
        } catch (ServiceException e2) {
            logger.error("Error while validating CVR number " + cvr + ". Will validate to false. ServiceException:", e2);
        } catch (Exception e3) {
            logger.error("Error while validating CVR number " + cvr + ". Will validate to false. Exception:", e3);
        }
        return result;
    }

}
