package dk.skat.ics.security;

import java.security.Principal;
import java.util.Set;

import javax.security.auth.Subject;

import weblogic.security.Security;


/**
 * ICS Security
 * IdentityInformation
 * <p/>
 * Based on the pattern(s) applied in the IdentityInformation class in IP.
 *
 * @author Christopher Derek Curry, KrisPoint A/S, cdcurry@krispoint.com
 * @since 1.0
 */
public class IdentityInformation {

    private String cvr;

    private String eori;

    private String cn;

    private String rid;

    /**
     * Check if the subject is an employee (working in a DK company).
     * @return true if user is an employee.
     */
    public boolean isEmployee() {
        return !(isSkatEmployee() || isEORIUser());
    }

    /**
     * Check if the subject is an EORI user (foreign company).
     * @return true if user is an EORI user.
     */
    public boolean isEORIUser() {
        return eori != null;
    }

    /**
     * NOTICE: THIS CHECK RELIES ON A CONSTANT VALUE EMBEDDED IN THE CODE!
     *
     * Check if the subject is a SKAT employee.
     * @return true if user is an employee.
     */
    public boolean isSkatEmployee() {
        return cvr.equals("19552101") && !isEORIUser();
    }

    /**
     * This method is invoked by the constructor to validate the values in the object before returning to the caller.
     * This is done to ensure that the contens of object is valid in a business sense.
     *
     * @throws IllegalStateException if the values cannot be accepted.
     */
    private void validateState() throws IllegalStateException {
        if (cn == null || cn.length() == 0) {
            throw new IllegalArgumentException("Argument 'cn' is null or is empty string.");
        }
        if (rid == null || rid.length() == 0) {
            throw new IllegalArgumentException("Argument 'rid' is null or is empty string.");
        }
    }

    /**
     * Create a new instance of a IdentityInformation object were all values is taken from the BEA WebLogic security context.
     *
     * @throws IllegalStateException if the context of the thread is invalid and do not contain a valid BEA WebLogic security context.
     */
    public IdentityInformation() throws IllegalStateException {
        Subject subject = Security.getCurrentSubject();
        if (subject == null) {
            throw new IllegalStateException("The weblogic.security.Security.getCurrentSubject() is null.");
        }

        Set<Principal> principals = subject.getPrincipals();
        this.cvr = ExtendedIdentityHelper.getCVRNumber(principals);
        this.eori = ExtendedIdentityHelper.getEORI(principals);
        this.cn = ExtendedIdentityHelper.getCN(principals);
        this.rid = ExtendedIdentityHelper.getRID(principals);
        validateState();
    }

    public String getCvr() {
        return cvr;
    }

    public String getEori() {
        return eori;
    }

    public String getCn() {
        return cn;
    }

    public String getRid() {
        return rid;
    }


    public String toString() {
        return "IdentityInformation{" +
                "cvr='" + cvr + '\'' +
                ", eori='" + eori + '\'' +
                ", cn='" + cn + '\'' +
                ", rid='" + rid + '\'' +
                ", isEmployee()='" + isEmployee() + '\'' +
                ", isEORIUser()='" + isEORIUser() + '\'' +
                ", isSkatEmployee()='" + isSkatEmployee() + '\'' +
                '}';
    }
}
