package dk.skat.ics.security;

import java.security.Principal;
import java.util.*;

import javax.security.auth.Subject;

import weblogic.security.Security;


/**
 * ICS Security
 * IdentityInformation
 * <p/>
 * Based on the pattern(s) applied in the IdentityInformation class in IP.
 *
 * @author Christopher Derek Curry, KrisPoint A/S, cdcurry@krispoint.com
 * @since 1.0
 */
public class IdentityInformation {

    private String cvr;

    private String eori;

    private String rid;

    private String authLevel;

    public static final String AUTH_LEVEL_4 = "AUTH_LEVEL_4";

    public static final String AUTH_LEVEL_6 = "AUTH_LEVEL_6";

    /**
     * Check if the subject is an employee (working in a DK company).
     *
     * @return true if user is an employee.
     */
    public boolean isEmployee() {
        return authLevel != null && authLevel.equals(AUTH_LEVEL_4) && !isEORIUser();
    }

    /**
     * Check if the subject is an EORI user (foreign company).
     *
     * @return true if user is an EORI user.
     */
    public boolean isEORIUser() {
        return eori != null;
    }

    /**
     * <p/>
     * Check if the subject is a SKAT employee.
     *
     * @return true if user is an employee.
     */
    public boolean isSkatEmployee() {
        return authLevel != null && authLevel.equals(AUTH_LEVEL_6) && !isEORIUser();
    }

    /**
     * This method validates the values in the object.
     * This is done to ensure that the contents of object is valid in a business sense.
     *
     * @throws IllegalStateException if the values cannot be accepted.
     */
    public void validateState() throws IllegalStateException {
        if (cvr == null || cvr.length() == 0) {
            throw new IllegalArgumentException("'cvr' is null or is empty string.");
        }
        if (rid == null || rid.length() == 0) {
            throw new IllegalArgumentException("'rid' is null or is empty string.");
        }
        if (authLevel == null || authLevel.length() == 0) {
            throw new IllegalArgumentException("'authLevel' is null or is empty string.");
        }
    }

    /**
     * Create a new instance of a IdentityInformation object were all values is taken from the BEA WebLogic security context.
     *
     * @throws IllegalStateException if the context of the thread is invalid and do not contain a valid BEA WebLogic security context.
     */
    public IdentityInformation() throws IllegalStateException {
        Subject subject = Security.getCurrentSubject();
        if (subject == null) {
            throw new IllegalStateException("The weblogic.security.Security.getCurrentSubject() is null.");
        }

        Set<Principal> principals = subject.getPrincipals();
        this.cvr = ExtendedIdentityHelper.getCVRNumber(principals);
        this.eori = ExtendedIdentityHelper.getEORI(principals);
        this.rid = ExtendedIdentityHelper.getRID(principals);
        this.authLevel = ExtendedIdentityHelper.getAuthLevel(principals);
    }

    public String getCvr() {
        return cvr;
    }

    public String getEori() {
        return eori;
    }

    public String getRid() {
        return rid;
    }


    public String toString() {
        return "IdentityInformation{" +
                "getCvr()='" + getCvr() + '\'' +
                ", getEori()='" + getEori() + '\'' +
                ", getRid()='" + getRid() + '\'' +
                ", isEmployee()='" + isEmployee() + '\'' +
                ", isEORIUser()='" + isEORIUser() + '\'' +
                ", isSkatEmployee()='" + isSkatEmployee() + '\'' +
                '}';
    }

}
