package dk.skat.ics.security;


import java.security.Principal;
import java.util.Set;


/**
 * ICS Security
 * Extended Identity Helper
 * <p/>
 * Based on the pattern(s) applied in the ExtendedIdentityHelper class in IP.
 *
 * @author Christopher Derek Curry, KrisPoint A/S, cdcurry@krispoint.com
 * @since 1.0
 */
public class ExtendedIdentityHelper {


    /** Prefix in the subject for the primary identity */
    public static final String ICS_IDENTITY_CVR = "CVR=";

    public static final String ICS_IDENTITY_CN = "CN=";

    /** Prefix in the subject for the primary identity */
    public static final String ICS_IDENTITY_RID = "RID=";

    public static final String ICS_IDENTITY_UID = "UID=";

    /** Prefix in the subject for the primary identity */
    public static final String ICS_IDENTITY_EORI = "EORI=";

    public static final String AUTH_LEVEL_PREFIX = "AUTH_LEVEL_";


    /**
     * Helper method that iterator through the given set of Principal object and returns the principal that is prefix
     * with identityInContextPrefix string
     *
     * @param principals Principals to be searched.
     * @param identityPrefix Principal object.
     * @return identified principal string without the prefix or null if none where found.
     * @throws IllegalArgumentException if the argument principals or identityPrefix is null.
     */
    private static Principal findPrincipal(Set<Principal> principals, String identityPrefix) throws IllegalArgumentException {
        if (principals == null)
            throw new IllegalArgumentException("The argument 'principals' cannot be null!");

        if (identityPrefix == null)
            throw new IllegalArgumentException("The argument 'identityPrefix' cannot be null!");

        for (Principal principal : principals)
            if (principal.getName().startsWith(identityPrefix))
                return principal;

        return null;
    }


    /**
     * Helper method that iterator through the given set of Principal object and returns the principal that is prefix with identityInContextPrefix string
     * @param principals to be searched.
     * @param identityPrefix string.
     * @return identified principal string without the prefix or null if none where found.
     * @throws IllegalArgumentException if the argument principals or identityInContextPrefix is null.
     */
    private static String findIdentity(Set<Principal> principals, String identityPrefix) throws IllegalArgumentException {
        Principal principal = findPrincipal(principals, identityPrefix);

        if (principal == null)
            return null;

        return principal.getName().substring(identityPrefix.length());
    }

    /**
     * Helper method that iterator through the given set of Principal object and returns the principal that is prefix with identityInContextPrefix string
     * @param principals to be searched.
     * @param identityPrefix string.
     * @return identified principal string without the prefix or null if none where found.
     * @throws IllegalArgumentException if the argument principals or identityInContextPrefix is null.
     */
    private static String findIdentityString(Set<Principal> principals, String identityPrefix) throws IllegalArgumentException {
        Principal principal = findPrincipal(principals, identityPrefix);

        if (principal == null)
            return null;

        return principal.getName();
    }


    /**
     * This method returns the common name found in the set of principals.
     * @param principals to be searched.
     * @return the common name
     * @throws IllegalArgumentException if principals is null or if identity is null.
     */
    public static String getCommonName(Set<Principal> principals) throws IllegalArgumentException {
        String identity = findIdentity(principals, ICS_IDENTITY_CN);
        return identity;
    }


    /**
     * This method returns the CVR number found in the set of principals.
     * @param principals to be searched.
     * @return the CVR number.
     * @throws IllegalArgumentException if principals is null or if identity is null.
     */
    public static String getCVRNumber(Set<Principal> principals) throws IllegalArgumentException {
        String identity = findIdentity(principals, ICS_IDENTITY_CVR);
/*        if (identity == null) {
            throw new IllegalStateException("The set of principal did not contain a " + ICS_IDENTITY_CVR + " identity!  Principals: " + principals);
        }
*/
        return identity;
    }

    /**
     * This method returns the RID found in the set of principals.
     * @param principals to be searched.
     * @return the RID.
     * @throws IllegalArgumentException if principals is null or if identity is null.
     */
    public static String getRID(Set<Principal> principals) throws IllegalArgumentException {
        String identity = findIdentity(principals, ICS_IDENTITY_RID);
      /*  if (identity == null) {
            throw new IllegalStateException("The set of principal did not contain a " + ICS_IDENTITY_RID + " identity!  Principals: " + principals);
        }
        */
        return identity;
    }

    /**
     * This method returns the UID found in the set of principals.
     * @param principals to be searched.
     * @return the UID.
     * @throws IllegalArgumentException if principals is null or if identity is null.
     */
    public static String getUID(Set<Principal> principals) throws IllegalArgumentException {
        String identity = findIdentity(principals, ICS_IDENTITY_UID);
        return identity;
    }

    /**
     * This method returns the RID found in the set of principals.
     * @param principals to be searched.
     * @return the RID.
     * @throws IllegalArgumentException if principals is null.
     */
    public static String getEORI(Set<Principal> principals) throws IllegalArgumentException {
        String identity = findIdentity(principals, ICS_IDENTITY_EORI);
        return identity;
    }

    /**
     * This method returns the AUTH_LEVEL_x found in the set of principals.
     * @param principals to be searched.
     * @return the RID.
     * @throws IllegalArgumentException if principals is null.
     */
    public static String getAuthLevel(Set<Principal> principals) throws IllegalArgumentException {
        String identity = findIdentityString(principals, AUTH_LEVEL_PREFIX);
        return identity;
    }

    /**
     * Factory method that returns a valid IdentityInformation that reflect the currently executing thread security context.
     *
     * @return a valid IdentityInformation object.
     * @throws IllegalStateException if not in the correct security context.
     */
    public static IdentityInformation getIdentityInformation() throws IllegalStateException {
        return new IdentityInformation();
    }    

    /**
     * Factory method that returns a valid IdentityInformation that reflect the currently executing thread security context.
     *
     * @return a valid IdentityInformation object.
     * @throws IllegalStateException if not in the correct security context.
     */
    public static IdentityInformation getIdentityInformation(Set<Principal> principals) throws IllegalStateException {
        return new IdentityInformation(principals);
    }
}
