package dk.skat.ics.security.authentication;

import java.security.cert.*;

import dk.skat.ics.security.ExtendedIdentityHelper;
import org.openoces.ooapi.utils.X509CertificateFactory;
import org.openoces.ooapi.utils.X509CertificatePropertyExtrator;
import weblogic.security.providers.authentication.UserNameMapper;
import weblogic.logging.NonCatalogLogger;

import java.util.*;
import java.util.regex.Pattern;
import java.util.regex.Matcher;

import javax.security.auth.x500.X500Principal;


/**
 * ICSX509UsernameMapper
 * <p/>
 * <p/>
 * <p/>
 * <p/>
 * Extracts information from MOCES and EORI certificates.
 * This mapper provides a
 *
 * @author Christopher Derek Curry, KrisPoint A/S, cdcurry@krispoint.com
 */
public class ICSX509UsernameMapper implements UserNameMapper {


    private static final NonCatalogLogger logger = new NonCatalogLogger(ICSX509UsernameMapper.class.getName());

    private final String ERROR_NO_CERTIFICATES_PROVIDED_NULL = "Paremeter 'certificates' cannot be null. Returning null.";

    private final String ERROR_NO_CERTIFICATES_PROVIDED_EMPTY = "The certificate array does not have any certificates - length equals zero. Returning null.";

    private final String AUTH_PROVIDER_NAMECALLBACK_PREFIX = "SKATEORI_";

    private static final Pattern MOCES_PATTERN = Pattern.compile(".*CVR:(\\d{8})\\-RID:(.+){1,47}.*");

    private static final Pattern MOCES_EORI_PATTERN = Pattern.compile(".*OU=EORI_.*");

    private static final String SYSTEMTESTII_CERT_POLICY_OID = "1.1.1.1.1.1.1.1.1.2";

    /**
     * Get configuration from JVM parameter
     *
     * @return
     */
    /*
    private static boolean disableOUSafeguardSetting() {
        String value = System.getProperty("ics.safeguard.missing.eori.ou");
        boolean result = true;
        if (value != null && value.equals("false")) {
            result = false;
        }
        return result;
    }
    */
    private static boolean icsRevocation() {
        String value = System.getProperty("ics.revocation");
        boolean result = true;
        if (value != null && value.equals("false")) {
            result = false;
        }
        return result;
    }

    private static final boolean checkRevocation = icsRevocation();

    //private static final boolean checkMissingOU = disableOUSafeguardSetting();

    /**
     * @see weblogic.security.providers.authentication.UserNameMapper#mapCertificateToUserName(java.security.cert.X509Certificate[], boolean)
     */
    public String mapCertificateToUserName(X509Certificate[] certificates, boolean ssl) {
        logger.debug("ICSX509UsernameMapper.mapCertificateToUserName - Enter");

        try {

            if (certificates == null) {
                logger.notice(ERROR_NO_CERTIFICATES_PROVIDED_NULL);
                return null;
            }

            if (certificates.length < 1) {
                logger.notice(ERROR_NO_CERTIFICATES_PROVIDED_EMPTY);
                return null;
            }

 //           printCertificates(certificates);
/*
            if (!(certificates.length >= 2)) {
                logger.notice("Cannot continue. At least two certificates required.");
                return null;
            }
*/
            List<X509Certificate> list = sortCertificatesIssuerLast(certificates);
            // Get the user's certificate
            X509Certificate certificate = list.get(0);
            // Get the CA
            X509Certificate signingCA = null;
            if (certificates.length >= 2) {
                 signingCA = list.get(1);
            }

            // Get encoded certificate
            byte[] encodedCertificate = certificate.getEncoded();

            X509Certificate encoded = X509CertificateFactory.getInstance().generateCertificate(encodedCertificate);

            X509Certificate generatedSigningCA = null;
            if (signingCA != null) {
                 generatedSigningCA = X509CertificateFactory.getInstance().
                    generateCertificate(signingCA.getEncoded());
            }

            // Get the subject string
            String subX509 = encoded.getSubjectDN().getName();

            logger.notice("Extracted Subject DN from certificate: " + encoded.getSubjectDN());

            // Rule A.1:
            if (!(checkMOCESCertificateType(encoded) || checkCUGCertificateType(encoded) ||
                    checkSystemTestIICertificateType(encoded))) {
                String warn = "Rule A.1 Violation: Policy not matched. Certificate does not qualify for further authentication. " +
                        "Will not process certificate with Subject DN :" + subX509;
                logger.warning(warn);
                return null;
            }

            // Rule A.2:
            if (checkRevocation) {
                boolean isRevoked = CachedRevocationChecker.check(encoded, generatedSigningCA);
                if (isRevoked) {
                    logger.notice("Rule A.2 Violation: Certificate is revoked. Will NOT facilitate login for: " + encoded.getSubjectDN());
                    return null;
                }
            } else {
                logger.warning("THIS SYSTEM IS AT RISK - CERTIFICATES ARE NOT CHECKED AGAINST CRL.");

            }

            // As we have passed rule A.1 and A.2 we know from the MOCES and CUG specification that there is a CVR
            // and RID field in the subject.
            String cvr = extractCVR(subX509);
            String rid = extractRID(subX509);
            String eori = null;

            // Rule A.3:
            if (checkCUGCertificateType(encoded) && checkEORIIdentitierCUGCertificateType(subX509)
                    && !cvr.equals("19552101")) {
                logger.warning("Rule A.3 Violation: Certificate does not qualify for further authentication." +
                        "Received certificate from a DanID Closed User Group realm that does not belong" +
                        "to this organization. This is an attempt to break in. Certificate Subject: " +
                        subX509);
                return null;
            }

            // Rule A.4:
            if (checkCUGCertificateType(encoded) &&
                    !checkEORIIdentitierCUGCertificateType(subX509)) {
                logger.notice("Rule A.4 Violation: Certificate does not qualify for further authentication." +
                        "Certificate does not include the expected ou=EORI_ identifier: " +
                        encoded.getSubjectDN());
                return null;
            }
            // Rule A.5:
            if (checkMOCESCertificateType(encoded) &&
                    checkEORIIdentitierCUGCertificateType(subX509)) {
                logger.notice("Rule A.5 Violation: Certificate does not qualify for further authentication." +
                        "Certificate included the ou=EORI_ identifier, but is not a CLOSED USER GROUP CERTIFICATE " +
                        encoded.getSubjectDN());
                return null;
            }

            // Rule A.6:
            if (checkEORIIdentitierCUGCertificateType(subX509)) {
                eori = extractEORI(subX509);
                if (!(eori != null && eori.length() > 0)) {
                    logger.notice("Rule A.6 Violation: Certificate does not qualify for further authentication." +
                            "Did not extract value from ou=EORI_ identifier: " +
                            encoded.getSubjectDN());
                    return null;
                }
            }

            // Build the "name" string that is to be passed on to the authentication plugin.
            String tempUser =
                    ExtendedIdentityHelper.ICS_IDENTITY_CVR + cvr + "," +
                            ExtendedIdentityHelper.ICS_IDENTITY_RID + rid;
            if (eori != null && eori.length() > 0) {
                tempUser = tempUser + "," + ExtendedIdentityHelper.ICS_IDENTITY_EORI + eori;
            }
            // Add prefix that will be accepted by authentication plugin.
            tempUser = AUTH_PROVIDER_NAMECALLBACK_PREFIX + tempUser;
            logger.notice("Produced new user string: " + tempUser);

            return tempUser;
        }
        catch (CertificateException e) {
            // Rule A.7
            logger.error("Rule A.7 Violation: " + e.getMessage());
            return null;
        } catch (Exception e) {
            // Rule A.7
            logger.error("Rule A.7 Violation: " + e.getMessage());
            return null;
        }
        finally {
            logger.debug("ICSX509UsernameMapper.mapCertificateToUserName - Leave");
        }
    }

    /**
     * @see weblogic.security.providers.authentication.UserNameMapper#mapDistinguishedNameToUserName(byte[])
     */
    public String mapDistinguishedNameToUserName(byte[] arg0) {
        // Not implemented.
        return null;
    }

    private boolean checkMOCESCertificateType(X509Certificate s) {
        String certificatePolicyOID = X509CertificatePropertyExtrator.getCertificatePolicyOID(s);
        if (certificatePolicyOID != null && certificatePolicyOID.startsWith("1.2.208.169.1.1.1.2")) {
            return true;
        } else {
            return false;
        }
    }

    private boolean checkCUGCertificateType(X509Certificate s) {
        String certificatePolicyOID = X509CertificatePropertyExtrator.getCertificatePolicyOID(s);
        if (certificatePolicyOID != null && certificatePolicyOID.equals("1.3.6.1.4.1.4386.2.2.1.1.1")) {
            // CERT original:  "1.3.6.1.4.1.31313.2.5.1.0"
            // CERT:           "1.3.6.1.4.1.4386.2.2.1.1.1"
            return true;
        } else {
            return false;
        }
    }

    private boolean checkSystemTestIICertificateType(X509Certificate s) {
        String certificatePolicyOID = X509CertificatePropertyExtrator.getCertificatePolicyOID(s);
        if (certificatePolicyOID != null && certificatePolicyOID.startsWith(SYSTEMTESTII_CERT_POLICY_OID)) {
            return true;
        } else {
            return false;
        }
    }


    private boolean checkMOCESCertificateType(String s) {
        Matcher matcher = MOCES_PATTERN.matcher(s);
        return matcher.matches();
    }

    private boolean checkEORIIdentitierCUGCertificateType(String s) {
        Matcher matcher = MOCES_EORI_PATTERN.matcher(s);
        return matcher.matches();
    }

    private String extractCN(String s) {
        Pattern pattern = Pattern.compile(".*CN=(.*?)\\+");
        Matcher m = pattern.matcher(s);
        m.find();
        String result = m.group(1);
        return result.trim();
    }

    private String extractCVR(String s) {
        Pattern pattern = Pattern.compile(".*CVR:(.*?)\\-");
        Matcher m = pattern.matcher(s);
        m.find();
        return m.group(1);
    }

    private String extractRID(String s) {
        Pattern pattern = Pattern.compile(".*RID:(.*?)\\,");
        Matcher m = pattern.matcher(s);
        m.find();
        return m.group(1);
    }

    private String extractEORI(String s) {
        Pattern pattern = Pattern.compile(".*OU=EORI_(.*?)\\,");
        Matcher m = pattern.matcher(s);
        m.find();
        return m.group(1);
    }


    private List<X509Certificate> sortCertificatesIssuerLast(List<X509Certificate> inputCertificates) {
        List<X509Certificate> result = new ArrayList<X509Certificate>(inputCertificates.size());
        Map<X500Principal, X509Certificate> certBySubject = new HashMap<X500Principal, X509Certificate>();
        for (X509Certificate certificate : inputCertificates) {
            certBySubject.put(certificate.getSubjectX500Principal(), certificate);
            boolean[] keyUsage = certificate.getKeyUsage();
            if (keyUsage != null && keyUsage[0] && !keyUsage[6]) {
                result.add(certificate);
            }
        }
        for (int i = 0; i < result.size(); i++) {
            X509Certificate certificate = result.get(i);
            X509Certificate issuer = certBySubject.get(certificate.getIssuerX500Principal());
            if (issuer != null && !result.contains(issuer)) {
                result.add(issuer);
            }
        }
        return result;
    }

    private List<X509Certificate> sortCertificatesIssuerLast(X509Certificate[] certificates) {
        List<X509Certificate> certlist = new ArrayList<X509Certificate>();
        for (X509Certificate certificate : certificates) {
            certlist.add(certificate);
        }
        return sortCertificatesIssuerLast(certlist);

    }

    private void printCertificates(X509Certificate[] certificates) {
        int i = 1;
        for (X509Certificate certificate : certificates) {
        String subject = certificate.getSubjectDN().getName();
        String issuer = certificate.getIssuerX500Principal().getName();

        String subX509 = certificate.getSubjectX500Principal().getName();
        logger.error("CERT: " + i + ": Reading certificates and printing specific values : ");
        logger.error("CERT: " + i + ":certificate.getSubjectDN().toString() : " + subject);
        logger.error("CERT: " + i + ":certificate.getSubjectX500Principal().getName() : [" + subX509 + "]");
        logger.error("CERT: " + i + ":certificate.getIssuerX500Principal().getName() : [" + issuer + "]");

        i++;
        }

    }


}
