package dk.skat.ics.security.authentication;

import java.security.cert.*;

import dk.skat.ics.security.ExtendedIdentityHelper;
import org.openoces.ooapi.utils.X509CertificateFactory;
import weblogic.security.providers.authentication.UserNameMapper;
import weblogic.logging.NonCatalogLogger;

import java.util.*;
import java.util.regex.Pattern;
import java.util.regex.Matcher;

import javax.security.auth.x500.X500Principal;


/**
 * ICSX509UsernameMapper
 * <p/>
 * Extracts information from MOCES and EORI certificates.
 * This mapper provides a
 *
 * @author Christopher Derek Curry, KrisPoint A/S, cdcurry@krispoint.com
 */
public class ICSX509UsernameMapper implements UserNameMapper {


    private static final NonCatalogLogger logger = new NonCatalogLogger(ICSX509UsernameMapper.class.getName());

    private final String ERROR_NO_CERTIFICATES_PROVIDED_NULL = "Paremeter 'certificates' cannot be null. Returning null.";

    private final String ERROR_NO_CERTIFICATES_PROVIDED_EMPTY = "The certificate array does not have any certificates - length equals zero. Returning null.";

    private final String AUTH_PROVIDER_NAMECALLBACK_PREFIX = "SKATEORI_";

    private static final Pattern MOCES_PATTERN = Pattern.compile(".*CVR:(\\d{8})\\-RID:(.+){1,47}.*");

    private static final Pattern MOCES_EORI_PATTERN = Pattern.compile(".*OU=EORI_.*");


    /**
     * @see weblogic.security.providers.authentication.UserNameMapper#mapCertificateToUserName(java.security.cert.X509Certificate[], boolean)
     */
    public String mapCertificateToUserName(X509Certificate[] certificates, boolean ssl) {
        logger.debug("ICSX509UsernameMapper.mapCertificateToUserName - Enter");
        try {

            if (certificates == null) {
                logger.error(ERROR_NO_CERTIFICATES_PROVIDED_NULL);
                return null;
            }

            if (certificates.length < 1) {
                logger.error(ERROR_NO_CERTIFICATES_PROVIDED_EMPTY);
                return null;
            }

            //printCertificates(certificates);

            List<X509Certificate> list = sortCertificatesIssuerLast(certificates);
            // Get first and only certificate
            X509Certificate certificate = list.get(0);

            // Get encoded certificate
            byte[] encodedCertificate = certificate.getEncoded();

            X509Certificate encoded = X509CertificateFactory.getInstance().generateCertificate(encodedCertificate);
            
            logger.notice("Extracted Subject DN from certificate: " + encoded.getSubjectDN());

            // Get the string that we will process
            String subX509 = encoded.getSubjectDN().getName();
            // Only process 1) standard MOCES certificates and 2) MOCES with OU=EORI_ extensions
            if (!(checkMOCESCertificateType(subX509) || checkMOCESEORICertificateType(subX509))) {
                String error = "Certificate does not qualify for this mapper. Will not process certificate with DN: " + subX509;
                logger.warning(error);
                // Now lets get us out of here.
                return null;
            }

            String cvr = extractCVR(subX509);
            String rid = extractRID(subX509);
            String eori = null;
            if (checkMOCESEORICertificateType(subX509)) {
                eori = extractEORI(subX509);

            }

            // Build the "name" string  that is to be passed on to the authentication plugin.
            String tempUser =
                    ExtendedIdentityHelper.ICS_IDENTITY_CVR + cvr + "," +
                    ExtendedIdentityHelper.ICS_IDENTITY_RID + rid;
            if (eori != null && eori.length() > 0) {
                tempUser = tempUser + "," + ExtendedIdentityHelper.ICS_IDENTITY_EORI + eori;
            } 
            // Add prefix that will be accepted by authentication plugin.
            tempUser = AUTH_PROVIDER_NAMECALLBACK_PREFIX + tempUser;
            logger.notice("Produced new user string: " + tempUser);

            return tempUser;
        }
        catch (CertificateException e) {
            logger.error(e.getMessage());
            return null;
        } catch (Exception e) {
            logger.error(e.getMessage());
            return null;
        }

        finally {
            logger.debug("ICSX509UsernameMapper.mapCertificateToUserName - Leave");
        }
    }

    /**
     * @see weblogic.security.providers.authentication.UserNameMapper#mapDistinguishedNameToUserName(byte[])
     */
    public String mapDistinguishedNameToUserName(byte[] arg0) {
        // Not implemented.
        return null;
    }

    private boolean checkMOCESCertificateType(String s) {
        Matcher matcher = MOCES_PATTERN.matcher(s);
        return matcher.matches();
    }

    private boolean checkMOCESEORICertificateType(String s) {
        Matcher matcher = MOCES_EORI_PATTERN.matcher(s);
        return matcher.matches();
    }

    private String extractCN(String s) {
        Pattern pattern = Pattern.compile(".*CN=(.*?)\\+");
        Matcher m = pattern.matcher(s);
        m.find();
        String result = m.group(1);
        return result.trim();
    }

    private String extractCVR(String s) {
        Pattern pattern = Pattern.compile(".*CVR:(.*?)\\-");
        Matcher m = pattern.matcher(s);
        m.find();
        return m.group(1);
    }

    private String extractRID(String s) {
        Pattern pattern = Pattern.compile(".*RID:(.*?)\\,");
        Matcher m = pattern.matcher(s);
        m.find();
        return m.group(1);
    }

    private String extractEORI(String s) {
        Pattern pattern = Pattern.compile(".*OU=EORI_(.*?)\\,");
        Matcher m = pattern.matcher(s);
        m.find();
        return m.group(1);
    }


    private List<X509Certificate> sortCertificatesIssuerLast(List<X509Certificate> inputCertificates) {
        List<X509Certificate> result = new ArrayList<X509Certificate>(inputCertificates.size());
        Map<X500Principal, X509Certificate> certBySubject = new HashMap<X500Principal, X509Certificate>();
        for (X509Certificate certificate : inputCertificates) {
            certBySubject.put(certificate.getSubjectX500Principal(), certificate);
            boolean[] keyUsage = certificate.getKeyUsage();
            if (keyUsage != null && keyUsage[0] && !keyUsage[6]) {
                result.add(certificate);
            }
        }
        for (int i = 0; i < result.size(); i++) {
            X509Certificate certificate = result.get(i);
            X509Certificate issuer = certBySubject.get(certificate.getIssuerX500Principal());
            if (issuer != null && !result.contains(issuer)) {
                result.add(issuer);
            }
        }
        return result;
    }

    private List<X509Certificate> sortCertificatesIssuerLast(X509Certificate[] certificates) {
        List<X509Certificate> certlist = new ArrayList<X509Certificate>();
        for (X509Certificate certificate : certificates) {
            certlist.add(certificate);
        }
        return sortCertificatesIssuerLast(certlist);

    }

    private void printCertificates(X509Certificate[] certificates) {
        for (X509Certificate certificate : certificates) {
            String subject = certificate.getSubjectDN().getName();
            String issuer = certificate.getIssuerX500Principal().getName();

            String subX509 = certificate.getSubjectX500Principal().getName();
            logger.error("Reading certificates and printing specific values : ");
            logger.error("certificate.getSubjectDN().toString() : " + subject);
            logger.error("certificate.getSubjectX500Principal().getName() : [" + subX509 + "]");
            logger.error("certificate.getIssuerX500Principal().getName() : [" + issuer + "]");
        }

    }

}
