package dk.skat.ics.security.authentication;

import net.sf.ehcache.Cache;
import net.sf.ehcache.CacheManager;
import net.sf.ehcache.Element;

import org.bouncycastle.asn1.ASN1InputStream;
import org.bouncycastle.asn1.DERIA5String;
import org.bouncycastle.asn1.DERObject;
import org.bouncycastle.asn1.DEROctetString;
import org.bouncycastle.asn1.x509.*;
import weblogic.logging.NonCatalogLogger;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.SignatureException;
import java.security.cert.*;
import java.util.ArrayList;
import java.util.List;

/**
 * CachedRevocationChecker
 *
 * <p/>
 *
 * Checks cert. against CRL provided in cert. Dist. points.
 * CRLs are cached.
 * 
 * @author Christopher Derek Curry, KrisPoint A/S, cdcurry@krispoint.com
 */
public class CachedRevocationChecker {


    private static final NonCatalogLogger logger = new NonCatalogLogger(CachedRevocationChecker.class.getName());

    private static final CacheManager cacheManager = CacheManager.create("ics-security.xml");

    private static Cache getCache() {
        String cacheName = "CRLCache";
        if (!cacheManager.cacheExists(cacheName)) {
            logger.notice("Creating CLR Cache...");
            Cache memoryOnlyCache = new Cache(cacheName, 100, false, false, 60 * 10, 60 * 10);
            cacheManager.addCache(memoryOnlyCache);
        }
        return cacheManager.getCache(cacheName);
    }


    private static X509CRL getCRL(String crlURLString) {
        X509CRL crl = null;
        if (getCache().isKeyInCache(crlURLString)) {
            logger.notice("CRL is in cache.");
            Element e = getCache().get(crlURLString);
            if (e != null) {
                crl = (X509CRL) e.getObjectValue();
            } else {
                logger.warning("Element expired or evicted. Getting CRL again.");
                crl = getCRLFromURL(crlURLString);
                if (crl != null) {
                    Element element = new Element(crlURLString, crl);
                    getCache().put(element);
                    logger.notice("CLR Cache elements: " + getCache().getSize());
                }
            }
        } else {
            logger.notice("Downloading CRL from " + crlURLString);
            crl = getCRLFromURL(crlURLString);
            if (crl != null) {
                Element element = new Element(crlURLString, crl);
                getCache().put(element);
                logger.notice("CLR Cache elements: " + getCache().getSize());
            }
        }
        return crl;
    }

    private static X509CRL getCRLFromURL(String crlURLString) {
        X509CRL crl = null;
        try {
            CertificateFactory certFactory = CertificateFactory.getInstance("X.509");
            URL crlUrl = new URL(crlURLString);
            InputStream crlStream = crlUrl.openStream();
            crl = (X509CRL) certFactory.generateCRL(crlStream);
        } catch (MalformedURLException e) {
            logger.error("Argument '" + crlURLString + "' malformed", e);
        } catch (CertificateException e) {
            logger.error("Got CertificateException. ", e);
        } catch (CRLException e) {
            logger.error("Got CRLException. ", e);
        } catch (IOException e) {
                logger.error("Cannot access CRL via URL: " + crlURLString + ". Please check your connection to this URL.", e);
        }
        return crl;
    }


    private CachedRevocationChecker() {
    }


    /**
     * Check cert.
     * @param certificate The certificate
     * @return true (certificate is revoked) or false (certificate is not revoked)
     */
     public static boolean check(X509Certificate certificate, X509Certificate generatedSigningCA) {
        try {
            List<String> urls = getCrlDistributionPoints(certificate);
            if (urls.size() == 0) {
                logger.notice("CRL Distribution point not found for " + certificate.getSubjectX500Principal() +
                        "Will respond as certificate was revoked.");
                return true;
            }
            String key = urls.get(0);
            logger.notice("Extracted revocation list from certificate: " + key);
            X509CRL crl = getCRL(key);
            if (crl != null) {

                if (!verifyCRL(crl, generatedSigningCA)) {
                    logger.notice("Cannot verify CRL as signing CA did not produce the CRL downloaded from: " + key +
                            ". As consequence the result will be the same as if the cert. is revoked.");
                    return true;
                } else {
                    logger.notice("CRL downloaded from " + key + " has been verified.");
                }

                boolean res = crl.isRevoked(certificate);
                logger.notice("Checking certificate against revocation list. Result = " + res);
                return res;
            }
        } catch (Exception e) {
            logger.error("Got exception. ", e);
        }
        return true;
    }


    /**
     * Check that the CRL was signed by the same CA that signed the Cert we are checking.
     * @param crl
     * @param generatedSigningCA
     * @return
     */
    private static boolean verifyCRL(X509CRL crl, X509Certificate generatedSigningCA) {
        boolean verified = false;
        try {
            crl.verify(generatedSigningCA.getPublicKey());
            verified = true;
        } catch (CRLException e) {
            logger.error("CRLException: ", e);
        } catch (NoSuchAlgorithmException e) {
            logger.error("NoSuchAlgorithmException: ", e);
        } catch (InvalidKeyException e) {
            logger.error("InvalidKeyException: ", e);
        } catch (NoSuchProviderException e) {
            logger.error("NoSuchProviderException: ", e);
        } catch (SignatureException e) {
        }
        return verified;
    }

    /**
     *
     * Extracts all CRL distribution point URLs from the "CRL Distribution Point"
     * extension in a X.509 certificate. If CRL distribution point extension is
     * unavailable, returns an empty list.
     *
     * Method taken from :
     *
     * http://www.nakov.com/blog/2009/12/01/x509-certificate-validation-in-java-build-and-verify-chain-and-verify-clr-with-bouncy-castle/
     */
    public static List<String> getCrlDistributionPoints(
            X509Certificate cert) throws CertificateParsingException, IOException {
        byte[] crldpExt = cert.getExtensionValue(
                X509Extensions.CRLDistributionPoints.getId());
        if (crldpExt == null) {
            List<String> emptyList = new ArrayList<String>();
            return emptyList;
        }
        ASN1InputStream oAsnInStream = new ASN1InputStream(
                new ByteArrayInputStream(crldpExt));
        DERObject derObjCrlDP = oAsnInStream.readObject();
        DEROctetString dosCrlDP = (DEROctetString) derObjCrlDP;
        byte[] crldpExtOctets = dosCrlDP.getOctets();
        ASN1InputStream oAsnInStream2 = new ASN1InputStream(
                new ByteArrayInputStream(crldpExtOctets));
        DERObject derObj2 = oAsnInStream2.readObject();
        CRLDistPoint distPoint = CRLDistPoint.getInstance(derObj2);
        List
                <String> crlUrls = new ArrayList<String>();
        for (DistributionPoint dp : distPoint.getDistributionPoints()) {
            DistributionPointName dpn = dp.getDistributionPoint();
            // Look for URIs in fullName
            if (dpn != null) {
                if (dpn.getType() == DistributionPointName.FULL_NAME) {
                    GeneralName[] genNames = GeneralNames.getInstance(
                            dpn.getName()).getNames();
                    // Look for an URI
                    for (int j = 0; j < genNames.length; j++) {
                        if (genNames[j].getTagNo() == GeneralName.uniformResourceIdentifier) {
                            String url = DERIA5String.getInstance(
                                    genNames[j].getName()).getString();
                            crlUrls.add(url);
                        }
                    }
                }
            }
        }
        return crlUrls;
    }

}
