package dk.skat.security.providers.atn;

import java.util.HashMap;

import javax.security.auth.login.AppConfigurationEntry;
import javax.security.auth.login.AppConfigurationEntry.LoginModuleControlFlag;

import weblogic.logging.NonCatalogLogger;
import weblogic.management.security.ProviderMBean;
import weblogic.security.provider.PrincipalValidatorImpl;
import weblogic.security.spi.*;

/**
 * DetachedIPSKATSystemAuthProvider
 *
 * @author Christopher Derek Curry, KrisPoint A/S, cdcurry@krispoint.com
 * @since 1.0
 *
 * See http://download.oracle.com/docs/cd/E12840_01/wls/docs103/dvspisec/atn.html
 */
public final class DetachedIPSKATSystemAuthProvider implements AuthenticationProvider, VersionableApplicationProvider {

    /**
     * Logger
     */
    private final static NonCatalogLogger logger = new NonCatalogLogger(DetachedIPSKATSystemAuthProvider.class.getName());

    /**
     * Description
     */
    private String description;

    /**
     * JAAS login control flag
     */
    private LoginModuleControlFlag controlFlag;


    private String skatSystemUserCN;

    private String skatSystemUserDN;

    private String providerPassword;

    /**
     * @see SecurityProvider#initialize(ProviderMBean, SecurityServices)
     */
    @SuppressWarnings({"EmptyTryBlock", "EmptyFinallyBlock"})
    public void initialize(ProviderMBean mbean, SecurityServices services) {
        try {
            logger.notice("Initializing DetachedIPSKATSystemAuthProvider");
            DetachedIPSKATSystemAuthProviderMBean myMBean = (DetachedIPSKATSystemAuthProviderMBean) mbean;
            description = mbean.getDescription();
            skatSystemUserCN = myMBean.getSKATSystemUserCN();
            skatSystemUserDN = myMBean.getSKATSystemUserDN();
            providerPassword = myMBean.getProviderPassword();
            logger.notice("skatSystemUserCN: " + skatSystemUserCN);
            logger.notice("skatSystemUserDN: " + skatSystemUserDN);
            logger.notice("providerPassword: ***encrypted***");
            String flag = myMBean.getControlFlag();
            if (flag.equalsIgnoreCase("REQUIRED")) {
                controlFlag = LoginModuleControlFlag.REQUIRED;
            } else if (flag.equalsIgnoreCase("OPTIONAL")) {
                controlFlag = LoginModuleControlFlag.OPTIONAL;
            } else if (flag.equalsIgnoreCase("REQUISITE")) {
                controlFlag = LoginModuleControlFlag.REQUISITE;
            } else if (flag.equalsIgnoreCase("SUFFICIENT")) {
                controlFlag = LoginModuleControlFlag.SUFFICIENT;
            } else {
                throw new IllegalArgumentException("Invalid flag value" + flag);
            }
        }
        catch (Exception e) {
            logger.error("Error: ", e);
            throw new RuntimeException(e);
        }
        finally {
        }
    }

    /**
     * @see SecurityProvider#shutdown()
     */
    @SuppressWarnings({"EmptyTryBlock", "EmptyFinallyBlock"})
    public void shutdown() {
        try {
        }
        finally {
        }
    }

    /**
     * @see SecurityProvider#getDescription()
     */
    @SuppressWarnings({"EmptyTryBlock", "EmptyFinallyBlock"})
    public String getDescription() {
        try {
            return description;
        }

        finally {
        }
    }


    /**
     * @see AuthenticationProvider#getLoginModuleConfiguration()
     */
    @SuppressWarnings({"EmptyTryBlock", "EmptyFinallyBlock"})
    public AppConfigurationEntry getLoginModuleConfiguration() {
        try {
            HashMap options = new HashMap();
            options.put("IdentityAssertion","false");
            options.put("skatSystemUserCN", skatSystemUserCN);
            options.put("skatSystemUserDN", skatSystemUserDN);
            options.put("providerPassword", providerPassword);
            return new AppConfigurationEntry(dk.skat.security.providers.atn.LoginModule.class.getName(), controlFlag, options);
        }
        finally {
        }
    }

    /**
     * @see AuthenticationProvider#getAssertionModuleConfiguration()
     */
    @SuppressWarnings({"EmptyTryBlock", "EmptyFinallyBlock"})
    public AppConfigurationEntry getAssertionModuleConfiguration() {
        try {
            HashMap options = new HashMap();
            options.put("IdentityAssertion","true");
            options.put("skatSystemUserCN", skatSystemUserCN);
            options.put("skatSystemUserDN", skatSystemUserDN);
            options.put("providerPassword", providerPassword);
            return new AppConfigurationEntry(dk.skat.security.providers.atn.LoginModule.class.getName(), controlFlag, options);
        }
        finally {
        }
    }

    /**
     * @see AuthenticationProvider#getPrincipalValidator()
     */
    @SuppressWarnings({"EmptyTryBlock", "EmptyFinallyBlock"})
    public PrincipalValidator getPrincipalValidator() {
        try {
            return new PrincipalValidatorImpl();
        }
        finally {
        }
    }

    /**
     * @see AuthenticationProvider#getIdentityAsserter()
     */
    @SuppressWarnings({"EmptyTryBlock", "EmptyFinallyBlock"})
    public IdentityAsserter getIdentityAsserter() {
        try {
            return null;
        }
        finally {
        }
    }

    /**
     * @see weblogic.security.spi.VersionableApplicationProvider#createApplicationVersion(java.lang.String, java.lang.String)
     */
    @SuppressWarnings({"EmptyTryBlock", "EmptyFinallyBlock"})
    public void createApplicationVersion(String appId, String sourceAppId) throws ApplicationVersionCreationException {
        try {
        }
        finally {
        }
    }

    /**
     * @see weblogic.security.spi.VersionableApplicationProvider#deleteApplication(java.lang.String)
     */
    @SuppressWarnings({"EmptyTryBlock", "EmptyFinallyBlock"})
    public void deleteApplicationVersion(String appId) throws ApplicationVersionRemovalException {
        try {
        }
        finally {
        }
    }

    /**
     * @see weblogic.security.spi.VersionableApplicationProvider#deleteApplicationVersion(java.lang.String)
     */
    @SuppressWarnings({"EmptyTryBlock", "EmptyFinallyBlock"})
    public void deleteApplication(String appName) throws ApplicationRemovalException {
        try {
        }
        finally {
        }
    }
}
