package dk.skat.security.providers.atn;

import java.io.IOException;
import java.security.Principal;
import java.util.*;

import javax.security.auth.Subject;
import javax.security.auth.callback.*;
import javax.security.auth.login.LoginException;

import weblogic.logging.NonCatalogLogger;
import weblogic.security.principal.WLSGroupImpl;
import weblogic.security.principal.WLSUserImpl;
import weblogic.security.spi.WLSUser;

/**
 * Login Module for DetachedIPSKATSystemAuthProvider
 *
 * @author Christopher Derek Curry, KrisPoint A/S, cdcurry@krispoint.com
 * @since 1.0
 *        <p/>
 *        See http://download.oracle.com/docs/cd/E12840_01/wls/docs103/dvspisec/atn.html
 */
final public class LoginModule implements javax.security.auth.spi.LoginModule {

    private static final NonCatalogLogger logger = new NonCatalogLogger(LoginModule.class.getName());

    /**
     * Subject that is to be authenticated.
     */
    private Subject subject;

    /**
     * Call back handler
     */
    private CallbackHandler callbackHandler;

    /**
     * Result of assertion/login process: Principal for this user.
     */
    private WLSUser wlsUser;

    private String skatSystemUserCN;

    private String skatSystemUserDN;

    private String providerPassword;

    private static final String nameCallBackPrompt = "username: ";

    private static final String passwordCallBackPrompt = "password";
          
    private ArrayList<WLSGroupImpl> listGroups = new ArrayList<WLSGroupImpl>();

    private boolean loginSucceeded;

    /**
     * Determine whether this is a assert identity job or not.
     */
    private boolean isIdentityAssertion;

    /**
     * @see javax.security.auth.spi.LoginModule#initialize(javax.security.auth.Subject, javax.security.auth.callback.CallbackHandler, java.util.Map, java.util.Map)
     */
    public void initialize(Subject subject, CallbackHandler callbackHandler, Map sharedState, Map options) {

        logger.notice("initialize!");
        this.subject = subject;
        this.callbackHandler = callbackHandler;
        isIdentityAssertion =
                "true".equalsIgnoreCase((String) options.get("IdentityAssertion"));
        skatSystemUserCN = (String) options.get("skatSystemUserCN");
        skatSystemUserDN = (String) options.get("skatSystemUserDN");
        providerPassword = (String) options.get("providerPassword");
    }

    /**
     * @see javax.security.auth.spi.LoginModule#login()
     */
    public boolean login() throws LoginException {

        if (isIdentityAssertion) {
            return assertLogin();
        } else {
            return authenticateLogin();
        }

    }

    /**
     * Authenticate login
     *
     * @return true if authentication went well
     *
     * @throws LoginException
     */
    private boolean authenticateLogin() throws LoginException {
        NameCallback nameCallback = new NameCallback(nameCallBackPrompt);
        PasswordCallback passwordCallback = new PasswordCallback(passwordCallBackPrompt, true);
        getCallback(new Callback[]{nameCallback, passwordCallback});

        String user = nameCallback.getName();
        String password = new String(passwordCallback.getPassword());
        assertEmptyString(user);
        assertEmptyString(password);

        if (!user.equals(this.skatSystemUserCN)) {
            loginSucceeded = false;
            return loginSucceeded;
        }
        // If we got the following line there is a match with user.
        if (!password.equals(this.providerPassword)) {
            loginSucceeded = false;
            return loginSucceeded;
        }
        String primaryIdentity = "PRIMARY_IDENTITY_" + this.skatSystemUserDN;
        listGroups.add(new WLSGroupImpl(primaryIdentity));
        listGroups.add(new WLSGroupImpl("AUTH_LEVEL_7"));
        wlsUser = new WLSUserImpl(this.skatSystemUserDN);
        loginSucceeded = true;
        logger.notice("Authenticated: " + user);
        return loginSucceeded;

    }

    /**
     * Assert user login.
     *
     * @return true if assertion went well 
     * @throws LoginException 
     */
    private boolean assertLogin() throws LoginException {
        NameCallback nameCallback = new NameCallback(nameCallBackPrompt);
        getCallback(new Callback[]{nameCallback});

        String user = nameCallback.getName();
        assertEmptyString(user);

        if (!user.equals(this.skatSystemUserCN)) {
            loginSucceeded = false;
            return loginSucceeded;
        }
        String primaryIdentity = "PRIMARY_IDENTITY_" + this.skatSystemUserDN;
        listGroups.add(new WLSGroupImpl(primaryIdentity));
        listGroups.add(new WLSGroupImpl("AUTH_LEVEL_7"));
        wlsUser = new WLSUserImpl(this.skatSystemUserDN);
        loginSucceeded = true;
        logger.notice("Asserted: " + user);
        return loginSucceeded;
    }


    /**
     * Simple helper method to assert strings.
     *
     * @param s String
     * @throws IllegalArgumentException If String is empty or NULL.
     */
    public static void assertEmptyString(String s) throws IllegalArgumentException {
        if ((s == null) || "".equals(s.trim())) {
            throw new IllegalArgumentException("The provided value cannot be null or empty.");
        }

    }

    /**
     * @see javax.security.auth.spi.LoginModule#commit()
     */
    public boolean commit() throws LoginException {
        try {
            if (loginSucceeded) {

                Set<Principal> principals = subject.getPrincipals();
                for (WLSGroupImpl s : listGroups) {
                    principals.add(s);
                    logger.debug("Adding group '" + s.getName() + "' for user '" + wlsUser.getName() + "'.");
                }
                principals.add(wlsUser);
            }
            logger.debug("Committed login for user '" + wlsUser + "'");
            return true;

        } catch (Exception e) {
            logger.error("Error: ", e);
            return false;
        }
    }

    /**
     * @see javax.security.auth.spi.LoginModule#abort()
     */
    public boolean abort() throws LoginException {
        try {
            Set<Principal> principals = subject.getPrincipals();

            principals.remove(wlsUser);
            for (WLSGroupImpl s : listGroups) {
                principals.remove(s);
            }
            return true;

        } catch (Exception e) {
            logger.error("Error: ", e);
            return false;
        }
    }

    /**
     * @see javax.security.auth.spi.LoginModule#logout()
     */
    public boolean logout() throws LoginException {
        try {
            Set<Principal> principals = subject.getPrincipals();

            principals.remove(wlsUser);
            for (WLSGroupImpl s : listGroups) {
                principals.remove(s);
            }
            return true;
        } catch (Exception e) {
            logger.error("Error: ", e);
            return false;
        }
    }


    private void getCallback(Callback... callbacks) throws LoginException {
        try {
            callbackHandler.handle(callbacks);
        } catch (IOException e) {
            throw new LoginException(e.getMessage());
        } catch (UnsupportedCallbackException e) {
            throw new LoginException(e.getMessage());
        }
    }

}
